import {fetchQuery} from "./contentUtility.js";
import Dataset from "./dataset";

const apiEndpoint = "/api/v1/datasets";
const baseURL = location.origin;
const defaultPagingValue = 20;
const lastQuery = {
    url: "",
    totalPages: 0,
    currentPage: 0
};

// definition of all buttons & sections
const addButton = document.getElementById("add-btn");
const filterButton = document.getElementById("filter-btn");
const searchButton = document.getElementById("search-btn");
const searchBar = document.getElementById("search-entry");
const sortButton = document.getElementById("sort-btn");
const resetButton = document.getElementById("reset-tools-btn");
const upvoteButtons = document.getElementsByClassName("upvote-btn");
const downvoteButtons = document.getElementsByClassName("downvote-btn");
export const searchSection = document.getElementById("search");
const recentSection = document.getElementById("recents");
const mostLikedSection = document.getElementById("top");

// ID of the timeout, because we need to cancel it at some point
export let searchBarTimeout;
const searchDelay = 500;

// Event listeners
addButton.addEventListener("click", () => {
    navigateToAdd();
});

filterButton.addEventListener("change", () => {
    const filterString = filterButton.value;
    if (filterString !== filterButton.querySelector("#default-filter").value) {
        fetchQuery(createQuery());
    }
});

searchButton.addEventListener("click", () => {
    fetchQuery(createQuery());

});

searchBar.addEventListener("input", () => {
    updateSections();
    clearTimeout(searchBarTimeout);
    searchBarTimeout = setTimeout(() => {
        fetchQuery(createQuery());
    }, searchDelay);
});

searchBar.addEventListener('keypress', function (e) {
    if (e.key === 'Enter') {
        fetchQuery(createQuery());
    }
});

sortButton.addEventListener("change", () => {
    fetchQuery(createQuery());
});

resetButton.addEventListener("click", () => {
    searchBar.value = "";
    filterButton.value = filterButton.querySelector("#default-filter").value;
    sortButton.value = sortButton.querySelector("#default-sort").value;
    updateSections();
});

// Consider moving this to datasets.js completely
const upvoteButtonClickListener = e => {
    const entryID = e.target.parentElement.parentElement.dataset.id;
    vote(entryID, true);
};
for (const upvoteButton of upvoteButtons) {
    upvoteButton.addEventListener("click", upvoteButtonClickListener);
}

// Consider moving this to datasets.js completely
const downvoteButtonClickListener = e  => {
    const entryID = e.target.parentElement.parentElement.dataset.id;
    vote(entryID, false);
};
for (const downvoteButton of downvoteButtons) {
    downvoteButton.addEventListener("click", downvoteButtonClickListener);
}

// functions of the main page
function navigateToAdd() {
    window.location.href = "/add";
}

function getFilterQuery() {
    let filterString= filterButton.value.toUpperCase();
    if (filterString === "NONE") {
        return ["type", "%"]
    } else if (document.querySelector('#filter-btn option:checked').parentElement.label === "Standard categories") {
        return ["type", filterString];
    } else {
        return ["category", filterString];
    }
}

function getSearchQuery() {
    let searchString = searchBar.value;
    return (searchString.length === 0 ? "%" : (searchString + "%"));
}

function getSortQuery() {
    let sortString = sortButton.value.toLowerCase().split(" ");
    if (sortString[1] === "a-z" || sortString[1] === "↑" || sortString[1] === "oldest-newest") {
        sortString[1] = "asc";
    } else {
        sortString[1] = "desc";
    }
    return sortString
}

// creates query for the whole toolbar, so that searching, sorting and filtering are always combined
function createQuery() {
    updateSections();
    let queryURL = new URL(apiEndpoint + "/search", baseURL);
    queryURL.searchParams.append("search", getSearchQuery());
    queryURL.searchParams.append(getFilterQuery()[0], getFilterQuery()[1]);
    queryURL.searchParams.append("sort", getSortQuery()[0]);
    queryURL.searchParams.append("direction", getSortQuery()[1]);
    queryURL.searchParams.append("size", defaultPagingValue.toString(10));
    return queryURL;
}

export function vote(entryID, up) {
    const fetchURL = new URL(
        `${apiEndpoint}/id/${entryID}/${up ? "up" : "down"}vote`,
        baseURL,
    );
    fetch(fetchURL, {
        method: "PUT",
        headers: {
            'Content-Type': 'application/json',
        }})
    .then(resp => resp.json())
    .then((data) => {
        console.log(data.upvotes); // TODO: remove, check einbauen: data.id === entryID?
    let dataset = document.querySelector('[data-id= ' + CSS.escape(entryID) + ']')
    dataset.querySelector("span").innerText = data.upvotes;
    });
}

function incrementPageCount() {
    lastQuery.currentPage++;
}

// updates the page display. If no query is present, the initial page is shown, otherwise the search results.
function updateSections() {
    if (searchBar.value === "" && sortButton.value === sortButton.querySelector("#default-sort").value
        && filterButton.value === filterButton.querySelector("#default-filter").value) {
        searchSection.classList.add("hidden");
        recentSection.classList.remove("hidden");
        mostLikedSection.classList.remove("hidden");
        resetButton.classList.add("hidden");
    } else {
        searchSection.classList.remove("hidden");
        recentSection.classList.add("hidden");
        mostLikedSection.classList.add("hidden");
        resetButton.classList.remove("hidden");
    }
}

// fetches the further categories used in the filter function
function fetchCategories() {
    const fetchURL = new URL(
        "api/v1/categories" , baseURL);
    fetch(fetchURL)
        .then(resp => resp.json())
        .then((data) => {
            for (let i = 0; i < data.length; i++) {
                let category = data[i].toLowerCase();
                category = category.charAt(0).toUpperCase() + category.slice(1);
                document.getElementById("other-categories").appendChild(new Option(category));
            }
        });
}

function fetchInitialEntries() {
    let topVotedQueryURL = new URL(apiEndpoint + "/search" + baseURL);
    topVotedQueryURL.searchParams.append("sort", getSortQuery()[0]);
    topVotedQueryURL.searchParams.append("direction", getSortQuery()[1]);
    topVotedQueryURL.searchParams.append("size", "1");
    fetch(topVotedQueryURL)
        .then(resp => resp.json())
        .then((data) => {
            document.querySelector("#top .datasets").appendChild(new Dataset(data[0]).createDatasetHTMLElement());
        });
}

window.onload = function () {
    fetchCategories();
    fetchInitialEntries();
    updateSections();
    if (searchBar.value !== "") {
        fetchQuery(createQuery());
    }
}
